/**
 * Tecnología del ocio
 * Bloque 1, unidad didáctica 10
 * Práctica 1
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <cwiid.h>

cwiid_err_t error; //Función para errores

int conmutabit(int in, int bit) {
	if ((bit & in) == 0) {
		return in | bit;
	}
	else {
		return in & ~bit;
	}
}

//Función para errores
void error(cwiid_wiimote_t *wiimote, const char *s, va_list ap) {
	if (wiimote != NULL) {printf("Error en el dispositivo %d: ", cwiid_get_id(wiimote));}
	vprintf(s, ap);
	printf("\n");
}

int main(int argc, char *argv[]) {

	cwiid_wiimote_t* wiimote; //Gestor del Wiimote
	bdaddr_t bdaddr; //Dirección Bluetooth del Wiimote
	struct cwiid_state estado; //Estado del Wiimote
	char tmp[256]; //Temporal para mostrar la dirección Bluetooth
	int salir = 0; //Control del bucle principal
	int estadoleds = 0; //Estado actual de los LEDs.
	int estadovib = 0; //Estado actual del vibrador
	char opcion = 0; //Opción pulsada

	cwiid_set_err(error); //Asigno la función de error

	printf("Poner el Wiimote en modo de descubrimiento (pulsar 1+2) y pulsar enter...\n");
	getchar();
	printf("Buscando...\n");
	bdaddr = *BDADDR_ANY; //Se aceptará la conexión de cualquier Wiimote
	if (!(wiimote = cwiid_connect(&bdaddr, 0))) { //Conexión
		printf("No se pudo detectar wiimote\n");
	}
	else {
		//Conexión establecida
		ba2str(&bdaddr, tmp); //Traducción a cadena de caracteres de la dirección Bluetooth
		printf("Dirección Bluetooth del Wiimote %i: %s\n", cwiid_get_id(wiimote), tmp);
		if (cwiid_set_rpt_mode(wiimote, CWIID_RPT_STATUS)) {
			printf("Error al establecer el modo de informe.\n");
			salir = 1; //Evita que entre en el bucle
		}
		while (!salir) {
			if (opcion != 10) {
				printf("Pulsa una de estas teclas y enter para...\n");
				printf("  1->Conmutar LED 1\n  2->Conmutar LED 2\n  3->Conmutar LED 3\n  4->Conmutar LED 4\n");
				printf("  5->Conmutar vibrador\n");
				printf("  e->Mostrar estado de LEDs y vibrador\n");
				printf("  x->Salir\n");
			}
			opcion = getchar();
			switch (opcion) {
				case '1':
					estadoleds = conmutabit(estadoleds, CWIID_LED1_ON);
					if (cwiid_command(wiimote, CWIID_CMD_LED, estadoleds)) {printf("Error al conmutar LED 1");}
					break;
				case '2':
					estadoleds = conmutabit(estadoleds, CWIID_LED2_ON);
					if (cwiid_command(wiimote, CWIID_CMD_LED, estadoleds)) {printf("Error al conmutar LED 2");}
					break;
				case '3':
					estadoleds = conmutabit(estadoleds, CWIID_LED3_ON);
					if (cwiid_command(wiimote, CWIID_CMD_LED, estadoleds)) {printf("Error al conmutar LED 3");}
					break;
				case '4':
					estadoleds = conmutabit(estadoleds, CWIID_LED4_ON);
					if (cwiid_command(wiimote, CWIID_CMD_LED, estadoleds)) {printf("Error al conmutar LED 4");}
					break;
				case '5':
					estadovib = 1- estadovib;
					if (cwiid_command(wiimote, CWIID_CMD_RUMBLE, estadovib)) {printf("Error al conmutar el vibrador");}
					break;
				case 'e':
					if (cwiid_get_state(wiimote, &estado) != 0) {
						printf("Error al obtener el estado del Wiimote\n");
					}
					else {
						if (estado.rpt_mode & CWIID_RPT_STATUS) {
							printf("LEDs: %i, Vibrador: %i\n", estado.led, estado.rumble);
						}
						else {
							printf("El mensaje recibido no es de estado general\n");
						}
					}
					break;
				case 'x':
					salir = 1;
					break;
			}
		}
		if (cwiid_disconnect(wiimote)) { //Desconexión del Wiimote
			printf("Error al desconectar el Wiimote\n");
		}
	}
	return 0;
}
