/**
 * Tecnología del ocio
 * Bloque 1, unidad didáctica 10
 * Práctica 2
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <cwiid.h>

cwiid_err_t error; //Función para errores

//Comienzo del trozo copiado de cwiid_internal.h
#define SEQ_LEN(seq) (sizeof(seq)/sizeof(struct write_seq))
#define RPT_SPEAKER_ENABLE		0x14
#define RPT_SPEAKER_DATA		0x18
#define RPT_SPEAKER_MUTE		0x19
#define SOUND_BUF_LEN 			21

enum write_seq_type {
	WRITE_SEQ_RPT,
	WRITE_SEQ_MEM
};

struct write_seq {
	enum write_seq_type type;
	uint32_t report_offset;
	const void *data;
	uint16_t len;
	uint8_t flags;
};
//Fin del trozo copiado de cwiid_internal.h

//Secuencia para inicializar el altavoz
struct write_seq speaker_enable_seq[] = {
	{WRITE_SEQ_RPT, RPT_SPEAKER_ENABLE, (const void *)"\x04", 1, 0},
	{WRITE_SEQ_RPT, RPT_SPEAKER_MUTE, (const void *)"\x04", 1, 0},
	//{WRITE_SEQ_MEM, 0xA20009, (const void *)"\x01", 1, CWIID_RW_REG},
	//{WRITE_SEQ_MEM, 0xA20001, (const void *)"\x08", 1, CWIID_RW_REG},
	{WRITE_SEQ_MEM, 0xA20001, (const void *)"\x00\x00\x00\x0e\x40\x00\x00", 7, CWIID_RW_REG},
	{WRITE_SEQ_MEM, 0xA20008, (const void *)"\x01", 1, CWIID_RW_REG},
	{WRITE_SEQ_RPT, RPT_SPEAKER_MUTE, (const void *)"\x00", 1, 0}
};

//Secuencia para deshabilitar el altavoz
struct write_seq speaker_disable_seq[] = {
	{WRITE_SEQ_RPT, RPT_SPEAKER_MUTE, (const void *)"\x04", 1, 0},
	{WRITE_SEQ_MEM, 0xA20009, (const void *)"\x00", 1, CWIID_RW_REG},
	{WRITE_SEQ_MEM, 0xA20001, (const void *)"\x00", 1, CWIID_RW_REG},
	{WRITE_SEQ_RPT, RPT_SPEAKER_ENABLE, (const void *)"\x00", 1, 0}
};

//Función que reproducirá el sonido
int cwiid_pitar(cwiid_wiimote_t *wiimote) {
	//Sonido a reproducir (pitido)
	unsigned char buf[SOUND_BUF_LEN] = { 0xA0, 0xCC, 0x33, 0xCC, 0x33, 0xCC, 0x33, 0xCC, 0x33, 0xCC, 0x33, 0xCC, 0x33, 0xCC, 0x33, 0xCC, 0x33, 0xCC, 0x33, 0xCC, 0x33};
	
	int i = 0, fallo = 0;

	//Inicialización del altavoz
	if (exec_write_seq(wiimote, SEQ_LEN(speaker_enable_seq), speaker_enable_seq)) {
		printf("Error al habilitar el altavoz\n");
	}
	else {
		while (i < 20 && !fallo) {//Reproducción múltiple del sonido para alargar su duración
			if (cwiid_send_rpt(wiimote, 0, RPT_SPEAKER_DATA, SOUND_BUF_LEN, buf)) {
				printf("Error al enviar secuencia en iteración nº %d\n", i);
				fallo = 1;
			}
			//Sería recomendable detener la ejecución durante algunos milisegundos en este punto
			i++;
		}
		//Deshabilitación del altavoz
		if (exec_write_seq(wiimote, SEQ_LEN(speaker_disable_seq), speaker_disable_seq)) {
			printf("Error al deshabilitar el altavoz\n");
		}
	}

	return fallo;
}

//Función para errores
void error(cwiid_wiimote_t *wiimote, const char *s, va_list ap) {
	if (wiimote != NULL) {printf("Error en el dispositivo %d: ", cwiid_get_id(wiimote));}
	vprintf(s, ap);
	printf("\n");
}

int main(int argc, char *argv[]) {

	cwiid_wiimote_t* wiimote; //Gestor del Wiimote
	bdaddr_t bdaddr; //Dirección Bluetooth del Wiimote
	char tmp[256]; //Temporal para mostrar la dirección Bluetooth

	cwiid_set_err(error); //Asigno la función de error

	printf("Poner el Wiimote en modo de descubrimiento (pulsar 1+2) y pulsar enter...\n");
	getchar();
	printf("Buscando...\n");
	bdaddr = *BDADDR_ANY; //Se aceptará la conexión de cualquier Wiimote
	if (!(wiimote = cwiid_connect(&bdaddr, 0))) { //Conexión
		printf("No se pudo detectar wiimote\n");
	}
	else {
		//Conexión establecida
		ba2str(&bdaddr, tmp); //Traducción a cadena de caracteres de la dirección Bluetooth
		printf("Dirección Bluetooth del Wiimote %i: %s\n", cwiid_get_id(wiimote), tmp);
		if (cwiid_pitar(wiimote)) {printf("Error al pitar\n");}
		if (cwiid_disconnect(wiimote)) { //Desconexión del Wiimote
			printf("Error al desconectar el Wiimote\n");
		}
	}
	return 0;
}
