/**
 * Tecnología del ocio
 * Bloque 1, unidad didáctica 10
 * Práctica 3
 */

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <cwiid.h>
#include <math.h>
#include <time.h>

cwiid_err_t error; //Función para errores

//Función que imprime patrones
//patron es el array con el patrón a imprimir
//lng es la longitud del patrón a imprimir
void imprime_patron(int patron[], int lng) {
	int i;
	printf("[ ");
	for (i = 0; i < lng; i++) {
		switch (patron[i]) {
			case 0:
				printf("UP");
				break;
			case 1:
				printf("DOWN");
				break;
			case 2:
				printf("A");
				break;
			case 3:
				printf("B");
				break;
		}
		printf(" ");
	}
	printf("]\n");
}

//Función para errores
void error(cwiid_wiimote_t *wiimote, const char *s, va_list ap) {
	if (wiimote != NULL) {printf("Error en el dispositivo %d: ", cwiid_get_id(wiimote));}
	vprintf(s, ap);
	printf("\n");
}

int main(int argc, char *argv[]) {

	cwiid_wiimote_t* wiimote; //Gestor del wiimote
	bdaddr_t bdaddr; //Dirección Bluetooth del wiimote
	char tmp[256]; //Temporal para mostrar la dirección Bluetooth
	struct timespec t, told; //Instante de obtención del mensaje actual y del mensaje anterior
	int mesg_count; //Número de mensajes obtenidos
	union cwiid_mesg *mesg; //Array de mensajes obtenido
	int i; //Índice del bucle que recorre los mensajes y elementos de patrones
	int j; //Índice auxiliar
	int salir = 0; //Control del bucle principal
	struct acc_cal calibracion; //Datos de calibración de Wiimote
	double accz; //Aceleración en el eje Z del Wiimote
	double accz_old; //Aceleración en el eje Z de Wiimote
	int patron[8], patron_usr[8]; //Patrón a reproducir y patrón reproducido por el usuario
	int lng_patron = 0, lng_patron_usr= 0; //Longitud útil del patrón a reproducir y longitud útil del patrón reproducido por el usuario
	int fallos = 0; //Fallos cometidos
	int hayfallo = 0; //Indicador de fallo

	cwiid_set_err(error); //Asigno la función de error

	printf("Poner el wiimote en modo de descubrimiento (pulsar 1+2) y pulsar enter...\n");
	getchar();
	printf("Buscando...\n");
	bdaddr = *BDADDR_ANY; //Se aceptará la conexión de cualquier wiimote
	if (!(wiimote = cwiid_connect(&bdaddr, CWIID_FLAG_MESG_IFC))) { //Conexión y habilitación de envío de mensajes
		printf("No se pudo detectar wiimote\n");
	}
	else {
		//Conexión establecida
		ba2str(&bdaddr, tmp); //Traducción a cadena de caracteres de la dirección Bluetooth
		printf("Dirección Bluetooth del wiimote %i: %s\n", cwiid_get_id(wiimote), tmp);
		srand(time(NULL)); //Inicializar la semilla aleatoria
		clock_gettime(CLOCK_REALTIME, &told);
		cwiid_set_rpt_mode(wiimote, CWIID_RPT_BTN|CWIID_RPT_ACC);
		if (cwiid_get_acc_cal(wiimote, CWIID_EXT_NONE, &calibracion)) {
			printf("No se pudieron obtener los datos de calibración del Wiimote\n");
		}
		else {
			printf("Pulsa el botón Home para salir\n");
			while (!salir) {
				if (lng_patron == 8) { //Fin de juego
					printf("Has completado el juego, ¡muy bien!\n");
					salir = 1;
				}
				else if (lng_patron == lng_patron_usr) { //El usuario ha ejecutado el número de gestos que debía
					if (lng_patron_usr > 0) { //Compruebo el patrón ejecutado
						i = 0;
						hayfallo = 0;
						while ((i < lng_patron_usr) && (!hayfallo)) { //Comparo los patrones
							if (patron_usr[i] != patron[i]) {
								printf("Debías reproducir: ");
								imprime_patron(patron, lng_patron);
								printf("Has reproducido  : ");
								imprime_patron(patron_usr, lng_patron_usr);
								hayfallo = 1;
							}
							i++;
						}
						if (hayfallo) { //Si hay fallo enciendo LEDs, muestro mensajes y activo vibrador
							fallos++;
							if (cwiid_command(wiimote, CWIID_CMD_LED, (int)pow(2, fallos-1))) {printf("Error al encender LED %d", fallos);}
							if (fallos < 4) {
								lng_patron_usr = 0;
								printf("Error, has fallado %d veces.\n", fallos);
							}
							else {
								printf("Has fallado cuatro veces, fin del juego.\n");
								salir = 1;
							}
							if (cwiid_command(wiimote, CWIID_CMD_RUMBLE, 1)) {printf("Error al activar vibrador");}
							sleep(1);
							if (cwiid_command(wiimote, CWIID_CMD_RUMBLE, 0)) {printf("Error al desactivar vibrador");}
							if (fallos < 4) {
								printf("Vuelve a probar:\n");
								imprime_patron(patron, lng_patron);
							}
						}
					}
					if (!hayfallo) { //Si no hay fallo añado un nuevo elemento al patrón
						patron[lng_patron] = rand() % 4;
						lng_patron++;
						printf("Patrón a reproducir: \n");
						imprime_patron(patron, lng_patron);
						lng_patron_usr = 0;
					}
				}
				if (!salir) {
					//Bloquear hasta obtener el mensaje
					if (cwiid_get_mesg(wiimote, &mesg_count, &mesg, &t) != 0) {
						printf("Error al obtener mensajes del wiimote\n");
					}
					else {
						for (i = 0; i < mesg_count; i++) {
							if (mesg[i].type == CWIID_MESG_ACC) { //Al recibir mensaje del acelerómetro
								//Calibrar las medidas
								accz = (double)(mesg[i].acc_mesg.acc[CWIID_Z] - calibracion.zero[CWIID_Z])/(double)(calibracion.one[CWIID_Z] - calibracion.zero[CWIID_Z]);
								if ((t.tv_sec-told.tv_sec) + (t.tv_nsec-told.tv_nsec)/1E9 > 0.5) { //Separación temporal entre gestos
									if (((accz/fabs(accz)) != (accz_old/fabs(accz_old))) && (fabs(accz-accz_old) > 0.8)) { //Obtención de posible gesto en eje Z
										if (accz > 0) {
											patron_usr[lng_patron_usr] = 1;
											lng_patron_usr++;
											printf("DOWN\n");
										}
										else {
											patron_usr[lng_patron_usr] = 0;
											lng_patron_usr++;
											printf("UP\n");
										}
										told = t;
									}
								}
								accz_old = accz;
							}
							else if (mesg[i].type == CWIID_MESG_BTN) {
								switch (mesg[i].btn_mesg.buttons) {
									case 4: //Botón B
										patron_usr[lng_patron_usr] = 3;
										lng_patron_usr++;
										printf("B\n");
										break;
									case 8: //Botón A
										patron_usr[lng_patron_usr] = 2;
										lng_patron_usr++;
										printf("A\n");
										break;
									case 128: //Botón Home
										salir = 1;
										break;
								}
							}
							else {
								printf("Tipo de mensaje no esperado\n");
							}
						}
					}
				}
			}
		}
		if (cwiid_disconnect(wiimote)) { //Desconexión del wiimote
			printf("Error al desconectar el wiimote\n");
		}
	}
	return 0;
}
